#!/usr/bin/env python3

"""
nwg-shell system tray screenshot applet
Copyright: 2023-2025 Piotr Miller & Contributors
e-mail: nwg.piotr@gmail.com
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project: https://github.com/nwg-piotr/nwg-shell
License: MIT
"""

import gi
import os
import subprocess
import signal
import sys

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib
from psutil import process_iter
from nwg_shell_config.tools import eprint, is_command, load_shell_data, load_json, check_key, get_data_dir
from enum import Enum

try:
    gi.require_version('AppIndicator3', '0.1')
    from gi.repository import AppIndicator3
except:
    eprint('libappindicator-gtk3 package not found - tray icon unavailable')

dir_name = os.path.dirname(__file__)
shell_data = load_shell_data()

settings_file = os.path.join(get_data_dir(), "settings")
settings = {}
if os.path.isfile(settings_file):
    settings = load_json(settings_file)
defaults = {
    "screenshot-applet-on": True
}
for key in defaults:
    check_key(settings, key, defaults[key])


voc = {}  # Vocabulary
nwg_system_update_arg = ""

_ind = None  # Indicator(object) containing

counter = 0
command = ""

if os.getenv("SWAYSOCK"):
    cmd_exec = "swaymsg exec"
elif os.getenv("HYPRLAND_INSTANCE_SIGNATURE"):
    cmd_exec = "hyprctl dispatch exec"
else:
    eprint("This program needs either sway or Hyprland environment")
    sys.exit(1)

if not is_command("screenshot"):
    eprint("'screenshot [fullscreen|region|focused|display|swappy]' command not found")
    sys.exit(1)


class ScreenshotType(Enum):
    FULLSCREEN = 1
    DISPLAY = 2
    FOCUSED = 3
    REGION = 4


def signal_handler(sig, frame):
    if sig == 2 or sig == 15:
        desc = {2: "SIGINT", 15: "SIGTERM"}
        eprint("screenshot-applet: terminated with {}".format(desc[sig]))
        Gtk.main_quit()
    elif sig != 17:
        print("screenshot-applet: signal {} received".format(sig))


def load_vocabulary():
    # We will only need several keys out of the global dictionary
    # Load basic vocabulary (en_US)
    global_voc = load_json(os.path.join(dir_name, "langs", "en_US.json"))
    if not global_voc:
        eprint("Failed loading vocabulary")
        sys.exit(1)

    # Detect lang (system or user-defined)
    lang = os.getenv("LANG").split(".")[0] if not shell_data["interface-locale"] else shell_data["interface-locale"]
    # Translate if necessary
    if lang != "en_US":
        loc_file = os.path.join(dir_name, "langs", "{}.json".format(lang))
        if os.path.isfile(loc_file):
            # Load localized vocabulary
            loc = load_json(loc_file)
            if not loc:
                eprint("Failed loading translation into '{}'".format(lang))
            else:
                # Replace keys w/ localized counterparts, if they exist
                for k in loc:
                    global_voc[k] = loc[k]

    # Select and return just the keys we need, in another dict
    global voc
    for k in ["screenshot", "screenshot-display", "screenshot-focused", "screenshot-fullscreen", "screenshot-region"]:
        if k in global_voc:
            voc[k] = global_voc[k]


def screenshot(item, s_type):
    global counter, command
    if s_type == ScreenshotType.FULLSCREEN:
        subprocess.Popen(f'{cmd_exec} screenshot fullscreen', shell=True)
    if s_type == ScreenshotType.DISPLAY:
        _ind.switch_icon(f'nwg-3', "")
        counter = 2
        command = f'{cmd_exec} screenshot display'
        GLib.timeout_add_seconds(1, count_down_and_execute)
    if s_type == ScreenshotType.FOCUSED:
        _ind.switch_icon(f'nwg-3', "")
        counter = 2
        command = f'{cmd_exec} screenshot focused'
        GLib.timeout_add_seconds(1, count_down_and_execute)
    elif s_type == ScreenshotType.REGION:
        subprocess.Popen(f'{cmd_exec} screenshot swappy', shell=True)


def menu():
    m = Gtk.Menu()

    item = Gtk.MenuItem.new_with_label(voc["screenshot-fullscreen"])
    item.connect('activate', screenshot, ScreenshotType.FULLSCREEN)
    m.append(item)

    item = Gtk.MenuItem.new_with_label(voc["screenshot-display"])
    item.connect('activate', screenshot, ScreenshotType.DISPLAY)
    m.append(item)

    item = Gtk.MenuItem.new_with_label(voc["screenshot-focused"])
    item.connect('activate', screenshot, ScreenshotType.FOCUSED)
    m.append(item)

    item = Gtk.MenuItem.new_with_label(voc["screenshot-region"])
    item.connect('activate', screenshot, ScreenshotType.REGION)
    m.append(item)

    item = Gtk.SeparatorMenuItem()
    m.append(item)

    m.show_all()
    return m


class Indicator(object):
    def __init__(self):
        self.item_update = None

        self.ind = AppIndicator3.Indicator.new('nwg_screenshot_applet', '',
                                               AppIndicator3.IndicatorCategory.APPLICATION_STATUS)

        self.ind.set_menu(menu())
        self.ind.set_title(voc["screenshot"])
        self.ind.set_icon_full("nwg-screenshot", voc["screenshot"])

    def switch_icon(self, icon, desc):
        self.ind.set_icon_full(icon, desc)


def count_down_and_execute():
    global counter, command, _ind
    if counter > 0:
        _ind.switch_icon(f'nwg-{counter}', "")
        counter -= 1
        # repeat
        return True
    else:
        cmd = command
        command = ""
        subprocess.Popen(cmd, shell=True)
        _ind.ind.set_icon_full("nwg-screenshot", voc["screenshot"])
        # do not repeat
        return False


def main():
    own_pid = os.getpid()
    # Interrupt running instances, if any
    for proc in process_iter():
        if "nwg-screenshot" in proc.name():
            pid = proc.pid
            if not pid == own_pid:
                eprint("Killing '{}', pid {}".format(proc.name(), pid))
                os.kill(pid, signal.SIGINT)

    # Set app_id
    GLib.set_prgname('nwg-screenshot')

    # Load localized dictionary
    load_vocabulary()

    global _ind
    _ind = Indicator()

    # Gentle termination; check updates on USR1 (for no reason / possible future use)
    catchable_sigs = set(signal.Signals) - {signal.SIGKILL, signal.SIGSTOP}
    for sig in catchable_sigs:
        signal.signal(sig, signal_handler)

    Gtk.main()


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

"""
nwg-shell system tray update indicator
Copyright: 2023-2025 Piotr Miller & Contributors
e-mail: nwg.piotr@gmail.com
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project: https://github.com/nwg-piotr/nwg-shell
License: MIT

NOTE: This script has been written to replace the 'updates' nwg-panel executor, which is Arch-specific.
It's quite well documented in comments, and expected to be adapted for use with other Linux distributions, too.

On Arch Linux and derivatives, the script depends on 'baph' (Basic AUR Package Helper) by Nate Maia
https://bitbucket.org/natemaia/baph, as well if it comes to checking updates, as to updating.

Updates are being performed by the `nwg-system-update` script. For your own distro, you will need your own
version of the latter, installed somewhere on $PATH.
"""
import time

import gi
import os
import subprocess
import signal
import sys
import threading

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GLib
from psutil import process_iter
from nwg_shell_config.tools import eprint, is_command, load_text_file, load_shell_data, load_json, check_key, \
    get_data_dir

try:
    gi.require_version('AppIndicator3', '0.1')
    from gi.repository import AppIndicator3
except:
    eprint('libappindicator-gtk3 package not found - tray icon unavailable')
    sys.exit(1)

dir_name = os.path.dirname(__file__)
shell_data = load_shell_data()

settings_file = os.path.join(get_data_dir(), "settings")
settings = {}
if os.path.isfile(settings_file):
    settings = load_json(settings_file)
defaults = {
    "update-indicator-on": True,
    "update-indicator-interval": 30,
    "update-command": "nwg-system-update"
}
for key in defaults:
    check_key(settings, key, defaults[key])

voc = {}  # Vocabulary
nwg_system_update_arg = ""

ind = None  # Indicator(object) containing


def signal_handler(sig, frame):
    if sig == 2 or sig == 15:
        desc = {2: "SIGINT", 15: "SIGTERM"}
        eprint("nwg-update-indicator: terminated with {}".format(desc[sig]))
        Gtk.main_quit()
    elif sig == 10:
        eprint("nwg-update-indicator: SIGUSR1 received, checking updates")
        if ind:
            ind.check_updates()
    elif sig != 17:
        print("nwg-update-indicator: signal {} received".format(sig))


def load_vocabulary():
    # We will only need several keys out of the global dictionary
    # Load basic vocabulary (en_US)
    global_voc = load_json(os.path.join(dir_name, "langs", "en_US.json"))
    if not global_voc:
        eprint("Failed loading vocabulary")
        sys.exit(1)

    # Detect lang (system or user-defined)
    lang = os.getenv("LANG").split(".")[0] if not shell_data["interface-locale"] else shell_data["interface-locale"]
    # Translate if necessary
    if lang != "en_US":
        loc_file = os.path.join(dir_name, "langs", "{}.json".format(lang))
        if os.path.isfile(loc_file):
            # Load localized vocabulary
            loc = load_json(loc_file)
            if not loc:
                eprint("Failed loading translation into '{}'".format(lang))
            else:
                # Replace keys w/ localized counterparts, if they exist
                for k in loc:
                    global_voc[k] = loc[k]

    # Select and return just the keys we need, in another dict
    global voc
    for k in ["you-are-up-to-date", "update", "updates", "check-updates", "exit", "press-enter-to-exit"]:
        if k in global_voc:
            voc[k] = global_voc[k]

    return voc


def check_distro():
    # This is just a skeleton function, and for now it detects Arch Linux only. Feel free to contribute.
    # Use unambiguous, lowercase name for your distro. You will need it in the Indicator.check_updates method.

    if os.path.isfile("/etc/os-release"):
        lines = load_text_file("/etc/os-release").splitlines()
        for line in lines:
            if line.startswith("NAME"):
                if "Arch" in line:
                    return "arch"
                elif "Venom" in line:
                    return "venom"
                # add elif for other distros

            if line.startswith("ID"):
                if "arch" in line:
                    return "arch"
                elif "venom" in line:
                    return "venom"
                # add elif for other distros

    elif os.path.isfile("/etc/lsb=release"):
        lines = load_text_file("/etc/lsb-release").splitlines()
        for line in lines:
            if line.startswith("DISTRIB_ID"):
                if "Arch" in line:
                    return "arch"
                # add elif for other distros

            if line.startswith("DISTRIB_DESCRIPTION"):
                if "Arch" in line:
                    return "arch"
                # add elif for other distros

    return ""


class Indicator(object):
    def __init__(self, distro):
        self.distro = distro
        self.item_update = None

        self.ind = AppIndicator3.Indicator.new('nwg_update_indicator', '',
                                               AppIndicator3.IndicatorCategory.APPLICATION_STATUS)

        self.ind.set_menu(self.menu())
        self.ind.set_title(voc["updates"])

        self.switch_icon("nwg-update-noupdate", "Checking")
        self.check_updates()

    def menu(self):
        menu = Gtk.Menu()

        self.item_update = Gtk.MenuItem.new_with_label(voc["update"])
        self.item_update.connect('activate', self.update)
        menu.append(self.item_update)

        item = Gtk.MenuItem.new_with_label(voc["check-updates"])
        item.connect('activate', self.check_updates)
        menu.append(item)

        item = Gtk.SeparatorMenuItem()
        menu.append(item)

        menu.show_all()
        return menu

    def check_updates(self, *args):
        self.switch_icon("nwg-update-checking", "Checking")

        thr = threading.Thread(target=self.check_updates_actual_job, args=(), kwargs={})
        thr.start()

        return True  # For this to be called periodically

    def check_updates_actual_job(self):
        update_details = ""
        # The code below should leave `update_details` string empty if no updates found.
        # Otherwise, it should set it as a short info, that will be appended to the 'Update' menu item.

        # Below we could add update check commands for other distros
        global nwg_system_update_arg
        if self.distro == "arch":
            nwg_system_update_arg = "-pacman"

            if is_command("baph"):
                nwg_system_update_arg = "-baph"
                eprint("Using baph")
                output = subprocess.check_output("baph -c".split()).decode('utf-8').strip()
                if output and output != "0 0":
                    u = output.split()
                    update_details = f"pacman: {u[1]}, AUR: {u[0]}"

            elif is_command("yay"):
                nwg_system_update_arg = "-yay"
                eprint("Using yay")
                pacman, aur = 0, 0
                try:
                    output = subprocess.check_output("checkupdates".split()).decode('utf-8').splitlines()
                    pacman = len(output)
                except subprocess.CalledProcessError:
                    pass
                try:
                    output = subprocess.check_output("yay -Qu".split()).decode('utf-8').splitlines()
                    aur = len(output)
                except subprocess.CalledProcessError:
                    pass

                if pacman or aur:
                    update_details = f"pacman: {pacman}, AUR: {aur}"

            else:
                eprint("Using checkupdates")
                pacman = 0
                try:
                    output = subprocess.check_output("checkupdates".split()).decode('utf-8').splitlines()
                    pacman = len(output)
                except subprocess.CalledProcessError:
                    pass

                if pacman:
                    update_details = f"pacman: {pacman}"

            if update_details:
                eprint(update_details)
            else:
                eprint(voc["you-are-up-to-date"])

        elif self.distro == "venom":
            nwg_system_update_arg = "-scratch"
            eprint("Using scratch")
            scratch = 0
            try:
                output = subprocess.check_output("venomupdates".split()).decode('utf-8').splitlines()
                scratch = len(output)
            except subprocess.CalledProcessError:
                pass

            if scratch:
                update_details = f"scratch: {scratch}"

        # elif self.distro == "something_else":
        #   place your code here

        if not update_details:
            GLib.timeout_add_seconds(0, self.switch_icon, "nwg-update-noupdate", voc["you-are-up-to-date"])
        else:
            GLib.timeout_add_seconds(0, self.switch_icon, "nwg-update-available", update_details)

    def update(self, *args):
        # Other distros: we already have the foot terminal installed.
        # You just need to provide your own `nwg-system-update` script somewhere on $PATH.
        # IMPORTANT: the presence of this script will also determine if to display corresponding
        # options in the config utility.
        if settings["update-command"] == "nwg-system-update":
            cmd = "exec foot nwg-system-update '{}'".format(voc["press-enter-to-exit"])
            if nwg_system_update_arg:
                cmd += f" {nwg_system_update_arg}"
            subprocess.call(cmd, shell=True)
        else:
            subprocess.call("exec {}".format(settings["update-command"]), shell=True)

        self.check_updates()

    def switch_icon(self, icon, desc):
        # self.ind.set_title(desc)
        if desc != voc["you-are-up-to-date"]:
            self.item_update.set_label("{} ({})".format(voc["update"], desc))
            self.item_update.show()
        else:
            self.item_update.hide()
        self.ind.set_icon_full(icon, desc)


def main():
    # Die if 'nwg-system-update` script absent
    if not is_command(settings["update-command"]):
        eprint("'nwg-system-update' script not found, terminating")
        sys.exit(1)

    own_pid = os.getpid()
    # Interrupt running instances, if any
    for proc in process_iter():
        if "nwg-update-ind" in proc.name():
            pid = proc.pid
            if not pid == own_pid:
                eprint("Killing '{}', pid {}".format(proc.name(), pid))
                os.kill(pid, signal.SIGINT)

    # Set app_id
    GLib.set_prgname('nwg-update-indicator')

    # Load localized dictionary
    global voc
    voc = load_vocabulary()

    distro = check_distro()
    if not distro:
        eprint("Couldn't determine the Linux distribution, terminating")
        sys.exit(1)
    else:
        eprint("nwg-update-indicator running on '{}'".format(distro))

    global ind
    ind = Indicator(distro)  # Will check updates for the 1st time in the constructor
    # Check periodically in given intervals
    GLib.timeout_add_seconds(settings["update-indicator-interval"] * 60, ind.check_updates)

    # Gentle termination; check updates on USR1 (for no reason / possible future use)
    catchable_sigs = set(signal.Signals) - {signal.SIGKILL, signal.SIGSTOP}
    for sig in catchable_sigs:
        signal.signal(sig, signal_handler)

    Gtk.main()


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

"""
This script uses the i3ipc python module to switch the layout splith / splitv
for the currently focused window, depending on its dimensions.
It works both on sway and i3 window managers.
Inspired by https://github.com/olemartinorg/i3-alternating-layout
Copyright: 2019-2025 Piotr Miller & Contributors
e-mail: nwg.piotr@gmail.com
Project: https://github.com/nwg-piotr/nwg-shell
License: MIT
Dependencies: python-i3ipc>=2.0.1 (i3ipc-python), python-psutil

This code is a version of the 'autotiling` script (https://github.com/nwg-piotr/nwg-shell),
modified for use with nwg-shell. I wanted to avoid adding the shell-specific stuff to the original script,
as it's quite widely used outside the project. All the arguments remain the same. One more dependency (psutil).

Additions:
- killing existing *autotiling* process instances on startup;
- gentle SIGINT & SIGTERM handler.
"""
import argparse
import os
import signal
import sys

from functools import partial
from psutil import process_iter
from i3ipc import Connection, Event

try:
    from .__about__ import __version__
except ImportError:
    __version__ = "unknown"

from nwg_shell_config.tools import temp_dir, get_data_dir, load_json, check_key

settings = load_json(os.path.join(get_data_dir(), "settings"))
check_key(settings, "autotiling-workspaces", "")
check_key(settings, "autotiling-output-limits", {})
check_key(settings, "autotiling-output-splitwidths", {})
check_key(settings, "autotiling-output-splitheights", {})


def save_string(string, file):
    try:
        file = open(file, "wt")
        file.write(string)
        file.close()
    except Exception as e:
        print(e)


def find_output_name(con):
    if con.type == "root":
        return None

    p = con.parent
    if p:
        if p.type == "output":
            return p.name
        else:
            return find_output_name(p)


def switch_splitting(i3, e, debug):
    try:
        con = i3.get_tree().find_focused()
        if con and not settings["autotiling-workspaces"] or (
                str(con.workspace().num) in settings["autotiling-workspaces"]):
            if con.floating:
                # We're on i3: on sway it would be None
                # May be 'auto_on' or 'user_on'
                is_floating = "_on" in con.floating
            else:
                # We are on sway
                is_floating = con.type == "floating_con"

            # Depth_limit contributed by @Syphdias to original autotiling script
            output_name = find_output_name(con)
            if settings["autotiling-output-limits"]:
                output_depth_limit = settings["autotiling-output-limits"][output_name] if output_name in settings[
                    "autotiling-output-limits"] else 0

                if output_depth_limit:
                    # Assume we reached the depth limit, unless we can find a workspace
                    depth_limit_reached = True
                    current_con = con
                    current_depth = 0
                    while current_depth < output_depth_limit:
                        # Check if we found the workspace of the current container
                        if current_con.type == "workspace":
                            # Found the workspace within the depth limitation
                            depth_limit_reached = False
                            break

                        # Look at the parent for next iteration
                        current_con = current_con.parent

                        # Only count up the depth, if the container has more than one container as child
                        if len(current_con.nodes) > 1:
                            current_depth += 1

                    if depth_limit_reached:
                        if debug:
                            print("Debug: Depth limit reached", file=sys.stderr)
                        return

            is_full_screen = con.fullscreen_mode == 1
            is_stacked = con.parent.layout == "stacked"
            is_tabbed = con.parent.layout == "tabbed"

            # Exclude floating containers, stacked layouts, tabbed layouts and full screen mode
            if (not is_floating
                    and not is_stacked
                    and not is_tabbed
                    and not is_full_screen):

                new_layout = "splitv" if con.rect.height > con.rect.width else "splith"

                if new_layout != con.parent.layout:
                    result = i3.command(new_layout)
                    if result[0].success and debug:
                        print("Debug: Switched to {}".format(new_layout), file=sys.stderr)
                    elif debug:
                        print("Error: Switch failed with err {}".format(result[0].error), file=sys.stderr, )

                # splitwidth & splitheight contributed by @JoseConseco to original autotiling script
                if e.change in ["new", "move"] and con.percent:
                    if con.parent.layout == "splitv":  # top / bottom
                        if output_name in settings["autotiling-output-splitheights"] and \
                                settings["autotiling-output-splitheights"][output_name] != 1.0:
                            i3.command("resize set height {} ppt".format(
                                int(con.percent * settings["autotiling-output-splitheights"][output_name] * 100)))
                            if debug:
                                print("Debug: Scaling height x {}".format(
                                    settings["autotiling-output-splitheights"][output_name]), file=sys.stderr)
                    else:  # left / right
                        if output_name in settings["autotiling-output-splitwidths"] and \
                                settings["autotiling-output-splitwidths"][output_name] != 1.0:
                            i3.command("resize set width {} ppt".format(
                                int(con.percent * settings["autotiling-output-splitwidths"][output_name] * 100)))
                            if debug:
                                print("Debug: Scaling width x {}".format(
                                    settings["autotiling-output-splitwidths"][output_name]), file=sys.stderr)

        elif debug:
            print("Debug: No focused container found or autotiling on the workspace turned off", file=sys.stderr)

    except Exception as e:
        print("Error: {}".format(e), file=sys.stderr)


def signal_handler(sig, frame):
    desc = {2: "SIGINT", 15: "SIGTERM"}
    if sig in [2, 15]:
        print("Terminated with {}".format(desc[sig]))
        sys.exit(0)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d",
                        "--debug",
                        action="store_true",
                        help="print debug messages to stderr")

    args = parser.parse_args()

    own_pid = os.getpid()

    for proc in process_iter():
        if "autotiling" in proc.name():
            pid = proc.pid
            if not pid == own_pid:
                print("Killing '{}', pid {}".format(proc.name(), pid))
                os.kill(pid, signal.SIGINT)

    catchable_sigs = set(signal.Signals) - {signal.SIGKILL, signal.SIGSTOP}
    for sig in catchable_sigs:
        signal.signal(sig, signal_handler)

    if args.debug:
        if settings["autotiling-workspaces"]:
            print("Debug: autotiling is only active on workspaces:", settings["autotiling-workspaces"], file=sys.stderr)
        if settings["autotiling-output-limits"]:
            print("Debug: per-output limits: {}".format(settings["autotiling-output-limits"]), file=sys.stderr)
        if settings["autotiling-output-splitwidths"]:
            print("Debug: per-output split widths: {}".format(settings["autotiling-output-splitwidths"]), file=sys.stderr)
        if settings["autotiling-output-splitheights"]:
            print("Debug: per-output split heights: {}".format(settings["autotiling-output-splitheights"]), file=sys.stderr)

    # For use w/ nwg-panel
    ws_file = os.path.join(temp_dir(), "autotiling")
    if settings["autotiling-workspaces"]:
        save_string(settings["autotiling-workspaces"], ws_file)
    else:
        if os.path.isfile(ws_file):
            os.remove(ws_file)

    handler = partial(switch_splitting, debug=args.debug)
    i3 = Connection()
    for e in ["WINDOW", "MODE"]:
        try:
            i3.on(Event[e], handler)
            print("{} subscribed".format(Event[e]))
        except KeyError:
            print("'{}' is not a valid event".format(e), file=sys.stderr)

    i3.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
This script automagically translates some preinstalled UI labels into users $LANG.
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project site: https://nwg-piotr.github.io/nwg-shell
Author's email: nwg.piotr@gmail.com
Copyright (c) 2021-2025 Piotr Miller & Contributors
License: MIT
"""

import os
import sys
import locale

from nwg_shell_config.tools import get_config_home, get_shell_data_dir, load_shell_data, load_json, save_json

config_home = get_config_home()
dir_name = os.path.dirname(__file__)
translation = {}


def main():
    shell_data_file = os.path.join(get_shell_data_dir(), "data")
    shell_data = load_shell_data()
    if not shell_data["autotranslated"]:
        print("Looks like auto-translation has not yet been done. Let's get this over with.")
        print("This will only be done once. Then you're free to modify labels manually.")

        user_locale = locale.getlocale()[0]

        langs_dir = os.path.join(dir_name, "autotranslate")
        translations = os.listdir(langs_dir)

        print(f"User locale: '{user_locale}'")
        if user_locale not in translations:
            print(f"Translation into '{user_locale}' not found. At least I tried.")
            shell_data["autotranslated"] = True
            save_json(shell_data, shell_data_file)
            sys.exit(0)
        else:
            print(f"Translation into '{user_locale}' found")
            global translation
            # en_US MUST contain all the keys, load them all just in case
            translation = load_json(os.path.join(dir_name, "autotranslate", "en_US"))
            user_lang = load_json(os.path.join(dir_name, "autotranslate", user_locale))
            # translate available words into users lang
            for key in translation:
                if key in user_lang:
                    translation[key] = user_lang[key]
            print(translation)

            # Translate panel configs
            config_dir = os.path.join(config_home, "nwg-panel")
            items = ["preset-0", "preset-1", "preset-2", "preset-3", "hyprland-0", "hyprland-1", "hyprland-2",
                     "hyprland-3"]
            for item in items:
                path = os.path.join(config_dir, item)
                panels = load_json(path)
                for panel in panels:
                    if "processes-label" in panel["controls-settings"]:
                        tr = translation["processes"]
                        panel["controls-settings"]["processes-label"] = tr

                    if "custom-items" in panel["controls-settings"]:
                        custom_items = panel["controls-settings"]["custom-items"]

                        for i in custom_items:
                            if i["name"] == "Wallpapers":
                                tr = translation["wallpapers"]
                                i["name"] = tr

                            if i["name"] == "GTK settings":
                                tr = translation["look-settings"]
                                i["name"] = tr

                            if i["name"] == "Wallpapers":
                                tr = translation["wallpapers"]
                                i["name"] = tr

                            if i["name"] == "Displays":
                                tr = translation["displays-settings"]
                                i["name"] = tr

                            if i["name"] == "Panel settings":
                                tr = translation["panel-settings"]
                                i["name"] = tr

                            if i["name"] == "Shell settings":
                                tr = translation["shell-settings"]
                                i["name"] = tr

                    if "menu" in panel["controls-settings"]:
                        menu = panel["controls-settings"]["menu"]
                        if menu["name"] == "Exit":
                            menu["name"] = translation["exit"]

                        entries = menu["items"]
                        for entry in entries:
                            if entry["name"] == "Lock":
                                entry["name"] = translation["lock-screen"]

                            if entry["name"] == "Exit sway session" or entry["name"] == "Exit Hyprland session" or \
                                    entry["name"] == "Logout":
                                entry["name"] = translation["exit-compositor"]

                            if entry["name"] == "Restart":
                                entry["name"] = translation["reboot"]

                            if entry["name"] == "Shutdown":
                                entry["name"] = translation["shutdown"]

                print(f"Saving {path}")
                save_json(panels, path)

            # Translate nwg-bar template (no longer in use)
            # config_dir = os.path.join(config_home, "nwg-bar")
            # for template in ["bar.json", "hyprland.json"]:
            #     path = os.path.join(config_dir, template)
            #
            #     bar = load_json(path)
            #     for item in bar:
            #         if item["label"] == "Lock" or item["label"] == "Loc_k":
            #             item["label"] = translation["lock"]
            #
            #         if item["label"] == "Logout" or item["label"] == "_Logout":
            #             item["label"] = translation["logout"]
            #
            #         if item["label"] == "Reboot" or item["label"] == "_Reboot":
            #             item["label"] = translation["reboot"]
            #
            #         if item["label"] == "Shutdown" or item["label"] == "_Shutdown":
            #             item["label"] = translation["shutdown"]
            #
            #     print(f"Saving {path}")
            #     save_json(bar, path)

            print("Marking autotranslation done.")
            shell_data["autotranslated"] = True
            save_json(shell_data, shell_data_file)

    else:
        print("Auto-translation already done, nothing to do.")
        sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
nwg-shell translation utility
Copyright: 2023-2025 Piotr Miller
e-mail: nwg.piotr@gmail.com
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project: https://github.com/nwg-piotr/nwg-shell
License: MIT
"""

import gi
import importlib
import locale
import os
import sys

from nwg_shell_config.tools import load_json, save_json, load_shell_data, eprint

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GLib

from importlib import util

dir_name = os.path.dirname(__file__)

# "entry_point": "command-name"
modules = {
    "nwg_shell_config": "nwg-shell-config",
    "nwg_panel": "nwg-panel",
    "nwg_displays": "nwg-displays",
    "nwg_clipman": "nwg-clipman",
    "azote": "Azote"
}
# supported apps, that aren't python modules
apps = ["nwg-look"]

existing_langs = []
keys = []
voc_en_us = {}
voc_user = {}
load_voc_user = True
app_in_edition = ""
user_locale = None
scrolled_window = None
translation_box = None
lang_hint_menu = None
existing_translations_label = None
keys_label = None
window = None


class Row(Gtk.Box):
    def __init__(self, key, voc_user):
        super().__init__()
        global voc_en_us
        self.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.set_spacing(6)
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.pack_start(vbox, True, True, 0)
        self.key = key
        self.voc_en_us = voc_en_us
        self.voc_user = voc_user

        lbl = Gtk.Label()
        lbl.set_markup('<b>"{}"</b>'.format(self.key))
        lbl.set_property("halign", Gtk.Align.START)
        vbox.pack_start(lbl, False, False, 0)

        lbl = Gtk.Label.new(self.voc_en_us[self.key])
        lbl.set_property("name", "original-text")
        lbl.set_selectable(True)
        lbl.set_line_wrap(True)
        lbl.set_property("halign", Gtk.Align.START)
        vbox.pack_start(lbl, False, False, 0)

        text_view = Gtk.TextView()
        text_view.set_property("name", "translation")
        self.text_buffer = text_view.get_buffer()
        translation = self.voc_user[self.key] if self.key in voc_user else ""
        self.text_buffer.set_text(translation)
        text_view.set_wrap_mode(Gtk.WrapMode.WORD)
        vbox.pack_start(text_view, False, False, 0)

        self.set_highlight()

        btn_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
        self.pack_start(btn_box, False, False, 0)

        self.btn_apply = Gtk.Button.new_from_icon_name("object-select", Gtk.IconSize.BUTTON)
        self.btn_apply.set_tooltip_text("Apply")
        self.btn_apply.connect("clicked", self.on_btn_apply)
        self.btn_apply.set_sensitive(False)
        btn_box.pack_end(self.btn_apply, False, True, 0)

        self.btn_restore = Gtk.Button.new_from_icon_name("edit-undo", Gtk.IconSize.BUTTON)
        self.btn_restore.set_tooltip_text("Restore")
        self.btn_restore.connect("clicked", self.on_btn_restore)
        self.btn_restore.set_sensitive(False)
        btn_box.pack_end(self.btn_restore, False, True, 0)

    def connect_textview(self):
        self.text_buffer.connect("changed", self.on_text_buffer_changed)

    def on_text_buffer_changed(self, text_buffer):
        start, end = text_buffer.get_bounds()
        text = text_buffer.get_text(start, end, True)
        if text:
            if self.key not in self.voc_user or text != self.voc_user[self.key]:
                self.btn_apply.set_sensitive(True)
            else:
                self.btn_apply.set_sensitive(False)
        else:
            self.btn_apply.set_sensitive(False)
        if self.key in self.voc_user and text != self.voc_user[self.key]:
            self.btn_restore.set_sensitive(True)
        else:
            self.btn_restore.set_sensitive(False)

    def on_btn_restore(self, btn):
        if self.key in self.voc_user and self.voc_user[self.key]:
            self.text_buffer.set_text(self.voc_user[self.key])
        else:
            self.text_buffer.set_text("")

    def on_btn_apply(self, btn):
        start, end = self.text_buffer.get_bounds()
        text = self.text_buffer.get_text(start, end, True)
        if text:
            self.voc_user[self.key] = text
            btn.set_sensitive(False)
            self.btn_restore.set_sensitive(False)
            self.set_highlight()

    def set_highlight(self):
        if self.key in self.voc_user and self.voc_user[self.key]:
            self.set_property("name", "row")
        else:
            self.set_property("name", "row-empty")


def build_translation_window():
    global scrolled_window
    if scrolled_window:
        scrolled_window.destroy()

    scrolled_window = Gtk.ScrolledWindow.new(None, None)
    scrolled_window.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled_window.set_propagate_natural_height(True)
    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
    scrolled_window.add(box)
    for key in keys:
        row = Row(key, voc_user)
        row.connect_textview()
        box.pack_start(row, False, False, 0)

    scrolled_window.show_all()
    translation_box.pack_start(scrolled_window, True, True, 0)

    window.show_all()


class LangHintMenu(Gtk.Menu):
    def __init__(self, text, valid_locales, entry):
        super().__init__()
        self.set_reserve_toggle_size(False)
        for loc in valid_locales:
            if text in loc:
                item = Gtk.MenuItem.new_with_label(loc)
                if loc in existing_langs:
                    item.set_property("name", "existing")
                item.connect("activate", set_entry_from_item, entry)
                self.append(item)
                self.show_all()


def set_entry_from_item(item, entry):
    entry.set_text(item.get_label())
    entry.grab_focus()


def validate_lang(entry, valid_locales, btn):
    global lang_hint_menu
    text = entry.get_text()
    if text in valid_locales:
        btn.set_sensitive(True)
    else:
        btn.set_sensitive(False)
        if len(text) >= 2:
            lang_hint_menu = LangHintMenu(text, valid_locales, entry)
            if len(lang_hint_menu.get_children()) > 0:
                lang_hint_menu.popup_at_widget(entry, Gdk.Gravity.NORTH, Gdk.Gravity.SOUTH, None)


def on_btn_select(btn, entry, fc_btn, combo):
    _locale = entry.get_text()
    global user_locale
    user_locale = _locale
    if _locale:
        load_dict_and_build_window(combo)
    fc_btn.unselect_all()


def on_btn_export(btn, ckb_ascii):
    global voc_user
    voc_user["_lang"] = user_locale
    voc_user["_module"] = app_in_edition
    save_json(voc_user, os.path.join(os.getenv("HOME"), "{}-{}.json".format(app_in_edition, user_locale)),
              en_ascii=ckb_ascii.get_active())


def on_file_chooser_button(fc_btn, entry_lang, combo):
    global voc_user
    voc_user = load_json(fc_btn.get_file().get_path())
    global user_locale
    if "_lang" in voc_user and "_module" in voc_user:
        user_locale = voc_user["_lang"]
        entry_lang.set_text(user_locale)
        # The "changed" signal must be emitted, even if we're setting the same id again.
        global load_voc_user
        load_voc_user = False
        combo.set_active_id(None)
        combo.set_active_id(voc_user["_module"])  # Triggers the load_dict_and_build_window() function.
    else:
        eprint("Internal '_lang' or '_module' key missing, couldn't load the file")


def load_dict_and_build_window(combo):
    if combo.get_active_id():
        global app_in_edition
        app_in_edition = combo.get_active_id()
        print("\n[Application: '{}']".format(app_in_edition))
        # some apps won't be python modules, so the path will be different
        exclusions = ["nwg-look"]  # store langs in `/usr/share/app_name/langs/` (t.b.c.)
        if app_in_edition not in exclusions:
            name = importlib.util.find_spec(combo.get_active_id()).origin
            parts = name.split("/")
            langs_dir = os.path.join("/".join(parts[:-1]), "langs")
        else:
            langs_dir = os.path.join("/usr/share", app_in_edition, "langs")
        print("langs_dir: {}".format(langs_dir))

        global existing_langs
        existing_langs = []
        for item in os.listdir(langs_dir):
            if item.endswith(".json"):
                existing_langs.append(item.split(".")[0])
        print("Existing langs:", existing_langs)
        existing_translations_label.set_markup(
            "<small><b>Existing langs</b>: {}</small>".format(", ".join(existing_langs)))

        # basic dictionary
        global voc_en_us
        voc_en_us = load_json(os.path.join(langs_dir, "en_US.json"))
        if voc_en_us:
            print("Default dict: en_US.json, {} keys".format(len(voc_en_us)))
        else:
            eprint("Couldn't load basic dictionary, exiting.")
            sys.exit(1)

        # basic dictionary keys
        global keys
        keys = []
        for key in voc_en_us:
            keys.append(key)
        keys.sort()

        global load_voc_user
        if load_voc_user:
            global voc_user
            voc_user = load_json(os.path.join(langs_dir, "{}.json".format(user_locale)))
            if voc_user:
                print("User dict: {}.json, {} keys".format(user_locale, len(voc_user)))
            else:
                voc_user = {}
                print("User lang '{}' does not yet exist, creating empty dictionary.".format(user_locale))

        load_voc_user = True

        build_translation_window()


def main():
    GLib.set_prgname('nwg-shell-translate')

    valid_locales = []
    for loc in os.listdir("/usr/share/i18n/locales/"):
        if not loc.startswith("translit") and "_" in loc:
            valid_locales.append(loc)
    print("{} valid locales found".format(len(valid_locales)))

    shell_data = load_shell_data()

    global user_locale
    user_locale = locale.getlocale()[0] if not shell_data["interface-locale"] else shell_data["interface-locale"]

    screen = Gdk.Screen.get_default()
    provider = Gtk.CssProvider()
    style_context = Gtk.StyleContext()
    style_context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
    css = b"""
            label#original-text { font-style: italic; padding: 2px }
            textview#translation { padding: 2px 2px 4px 2px }
            box#row { margin: 0 12px 0 0; padding: 6px; border: solid 1px }
            box#row-empty { margin: 0 12px 0 0; padding: 6px; border: solid 2px; border-color: #F00 }
            menuitem#existing { font-weight: bold; color: #0c6 }
            """
    provider.load_from_data(css)

    global window
    window = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
    window.connect('destroy', Gtk.main_quit)

    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
    box.set_property("margin", 6)
    window.add(box)

    app_select_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
    box.pack_start(app_select_box, False, False, 6)
    lbl = Gtk.Label.new("Application:")
    app_select_box.pack_start(lbl, False, False, 0)

    combo = Gtk.ComboBoxText()
    app_select_box.pack_start(combo, False, False, 0)
    for key in modules:
        try:
            # Older nwg-shell components' versions may have no valid langs. We don't want them in the combo.
            name = importlib.util.find_spec(key).origin
            parts = name.split("/")
            base_dict = os.path.join("/".join(parts[:-1]), "langs", "en_US.json")
            if os.path.isfile(base_dict):
                combo.append(key, modules[key])
            else:
                eprint("Skipping '{}', 'en_US.json' file not found.".format(key))
        except:
            eprint("Couldn't find path for module '{}'".format(key))

    for app in apps:
        if os.path.isfile(os.path.join("/usr/share", app, "langs", "en_US.json")):
            combo.append(app, app)

    global existing_translations_label
    existing_translations_label = Gtk.Label()
    existing_translations_label.set_line_wrap_mode(Gtk.WrapMode.WORD)
    app_select_box.pack_start(existing_translations_label, False, False, 0)

    global translation_box
    translation_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
    translation_box.set_property("expand", True)
    box.pack_start(translation_box, True, True, 0)

    button_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
    button_box.set_property("margin", 6)
    box.pack_start(button_box, False, False, 0)

    img = Gtk.Image.new_from_icon_name("nwg-shell-translate", Gtk.IconSize.LARGE_TOOLBAR)
    button_box.pack_start(img, False, False, 0)
    lbl = Gtk.Label()
    lbl.set_markup("<b>nwg-shell-translate</b> | lang:".format(user_locale))
    button_box.pack_start(lbl, False, False, 0)

    btn_select = Gtk.Button.new_with_label("Select")
    btn_select.set_tooltip_text("Opens current dictionary for entered locale,\nif it exists. Otherwise - "
                                "creates an empty dictionary")

    fc_btn = Gtk.FileChooserButton.new("Open", Gtk.FileChooserAction.OPEN)

    entry_lang = Gtk.Entry()
    entry_lang.set_width_chars(10)
    entry_lang.set_text(user_locale)
    entry_lang.connect("changed", validate_lang, valid_locales, btn_select)
    button_box.pack_start(entry_lang, False, False, 0)

    button_box.pack_start(btn_select, False, False, 0)
    btn_select.connect("clicked", on_btn_select, entry_lang, fc_btn, combo)

    fc_btn.set_current_folder(os.getenv("HOME"))
    fc_btn.set_width_chars(10)
    fc_btn.set_tooltip_text("Opens your previously saved translation")
    f_filter = Gtk.FileFilter()
    f_filter.set_name("Translation files (*_??.json)")
    f_filter.add_pattern("*_??.json")
    fc_btn.connect("file-set", on_file_chooser_button, entry_lang, combo)
    fc_btn.add_filter(f_filter)
    button_box.pack_start(fc_btn, False, False, 0)

    ckb_ascii = Gtk.CheckButton.new()
    ckb_ascii.set_label("ASCII")
    ckb_ascii.set_tooltip_text("Replace accented characters with ASCII codes.")
    button_box.pack_end(ckb_ascii, False, False, 0)

    btn = Gtk.Button.new_with_label("Export")
    btn.set_tooltip_text("Exports current translation to a .json file in you $HOME directory.\n"
                         "You can then email it to nwg.piotr@gmail.com or share in another way.\n"
                         "E.g. gist.github.com is always a good idea.")
    btn.connect("clicked", on_btn_export, ckb_ascii)

    button_box.pack_end(btn, False, False, 0)

    btn = Gtk.Button.new_with_label("Close")
    btn.set_tooltip_text("Exits the program")
    btn.connect('clicked', Gtk.main_quit)
    button_box.pack_end(btn, False, False, 0)

    combo.connect("changed", load_dict_and_build_window)
    combo.set_active_id("nwg_shell_config")

    window.show_all()
    Gtk.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
nwg-shell config utility
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project site: https://nwg-piotr.github.io/nwg-shell
Author's email: nwg.piotr@gmail.com
Copyright (c) 2021-2025 Piotr Miller & Contributors
License: MIT
"""

import argparse
import signal
import time

from nwg_shell_config.tools import *
from nwg_shell_config.ui_components import *
from nwg_shell_config.__about__ import __version__, __need_update__
import gi

gi.require_version('Gdk', '3.0')
from gi.repository import Gdk, GLib

gi.require_version('Gtk', '3.0')

dir_name = os.path.dirname(__file__)

shell_data = load_shell_data()
pending_updates = 0
update_btn = Gtk.Button()

data_dir = ""
config_home = get_config_home()

data_home = get_data_home()

voc = {}
ui = None

outputs = []
settings = {}
preset_0 = {}
preset_1 = {}
preset_2 = {}
preset_3 = {}
preset_custom = {}

content = Gtk.Frame()
submenus = []
current_submenu = None
btn_apply = Gtk.Button()
grid = Gtk.Grid()

backup_configs = ["azote", "foot", "gtklock", "hypr", "nwg-bar", "nwg-displays", "nwg-dock", "nwg-dock-hyprland",
                  "nwg-drawer", "nwg-look", "nwg-panel", "sway", "swaync"]

backup_data = ["nwg-look", "nwg-panel", "nwg-shell-config"]


def check_updates():
    global shell_data
    shell_data = load_shell_data()
    global pending_updates
    pending_updates = 0
    for v in __need_update__:
        if v not in shell_data["updates"] and is_newer(v, shell_data["installed-version"]):
            pending_updates += 1
    global update_btn
    if pending_updates > 0:
        img = Gtk.Image.new_from_icon_name("nwg-shell-update", Gtk.IconSize.DIALOG)

        update_btn.set_label("Updates ({})".format(pending_updates))
    else:
        img = Gtk.Image.new_from_icon_name("nwg-shell", Gtk.IconSize.DIALOG)
        update_btn.set_label("Updates")
    update_btn.set_image(img)


def signal_handler(sig, frame):
    if sig == 2 or sig == 15:
        desc = {2: "SIGINT", 15: "SIGTERM"}
        print("terminated with {}".format(desc[sig]))
        Gtk.main_quit()
    elif sig == 10:
        print("SIGUSR1 received, checking updates")
        check_updates()
    elif sig != 17:
        print("signal {} received".format(sig))


def validate_workspaces(gtk_entry):
    valid_text = ""
    for char in gtk_entry.get_text():
        if char.isdigit() or char == " ":
            valid_text += char
    while '  ' in valid_text:
        valid_text = valid_text.replace('  ', ' ')
    gtk_entry.set_text(valid_text)


def handle_keyboard(window, event):
    if event.type == Gdk.EventType.KEY_RELEASE and event.keyval == Gdk.KEY_Escape:
        window.close()


def side_menu():
    list_box = Gtk.ListBox()
    list_box.set_property("margin-top", 6)

    row = Gtk.ListBoxRow()
    row.set_selectable(False)
    lbl = Gtk.Label()
    lbl.set_property("halign", Gtk.Align.START)
    lbl.set_markup("<b>{}</b>".format(voc["common"]))
    lbl.set_property("margin-top", 3)
    lbl.set_property("margin-start", 3)
    row.add(lbl)
    list_box.add(row)

    row = SideMenuRow(voc["screen-settings"])
    row.eb.connect("button-press-event", set_up_screen_tab)
    list_box.add(row)

    row = SideMenuRow(voc["autotiling"])
    row.eb.connect("button-press-event", set_up_autotiling_tab)
    list_box.add(row)

    row = SideMenuRow(voc["keyboard"])
    row.eb.connect("button-press-event", set_up_keyboard_tab)
    list_box.add(row)

    row = SideMenuRow(voc["pointer-device"])
    row.eb.connect("button-press-event", set_up_pointer_tab)
    list_box.add(row)

    row = SideMenuRow(voc["touchpad"])
    row.eb.connect("button-press-event", set_up_touchpad_tab)
    list_box.add(row)

    row = SideMenuRow(voc["idle-lock-screen"])
    row.eb.connect("button-press-event", set_up_lockscreen_tab)
    list_box.add(row)

    row = SideMenuRow(voc["gtklock"])
    row.eb.connect("button-press-event", set_up_gtklock_tab)
    list_box.add(row)

    row = SideMenuRow(voc["applications"])
    row.eb.connect("button-press-event", set_up_applications_tab)
    list_box.add(row)

    row = SideMenuRow(voc["backup"])
    row.eb.connect("button-press-event", set_up_backup_tab, config_home, data_home, backup_configs, backup_data)
    list_box.add(row)

    row = SideMenuRow(voc["system-info"])
    row.eb.connect("button-press-event", set_up_sys_info_tab)
    list_box.add(row)

    row = Gtk.ListBoxRow()
    row.set_selectable(False)
    lbl = Gtk.Label()
    lbl.set_property("halign", Gtk.Align.START)
    lbl.set_property("margin-top", 6)
    lbl.set_markup("<b>{}</b>".format(voc["desktop-styles"]))
    lbl.set_property("margin-start", 3)
    row.add(lbl)
    list_box.add(row)

    row = SideMenuRow("{} 0".format(voc["preset"]))
    list_box.add(row)

    submenu_0 = preset_menu(0)
    list_box.add(submenu_0)
    row.eb.connect("button-press-event", toggle_submenu, submenu_0)

    row = SideMenuRow("{} 1".format(voc["preset"]))
    list_box.add(row)

    submenu_1 = preset_menu(1)
    list_box.add(submenu_1)
    row.eb.connect("button-press-event", toggle_submenu, submenu_1)

    row = SideMenuRow("{} 2".format(voc["preset"]))
    list_box.add(row)

    submenu_2 = preset_menu(2)
    list_box.add(submenu_2)
    row.eb.connect("button-press-event", toggle_submenu, submenu_2)

    row = SideMenuRow("{} 3".format(voc["preset"]))
    list_box.add(row)

    submenu_3 = preset_menu(3)
    list_box.add(submenu_3)
    row.eb.connect("button-press-event", toggle_submenu, submenu_3)

    row = SideMenuRow(voc["custom-preset"])
    list_box.add(row)

    submenu_c = preset_menu("c")
    list_box.add(submenu_c)
    row.eb.connect("button-press-event", toggle_submenu, submenu_c)

    list_box.set_selection_mode(Gtk.SelectionMode.NONE)
    return list_box


def preset_menu(preset_id):
    if preset_id == 0:
        preset = preset_0
        preset_name = "{} 0".format(voc["preset"])
    elif preset_id == 1:
        preset = preset_1
        preset_name = "{} 1".format(voc["preset"])
    elif preset_id == 2:
        preset = preset_2
        preset_name = "{} 2".format(voc["preset"])
    elif preset_id == 3:
        preset = preset_3
        preset_name = "{} 3".format(voc["preset"])
    else:
        preset = preset_custom
        preset_name = "Custom preset"

    list_box = Gtk.ListBox()

    row = SideMenuRow(voc["app-drawer"], margin_start=18)
    row.eb.connect("button-press-event", set_up_drawer_tab, preset, preset_name)
    list_box.add(row)

    row = SideMenuRow(voc["dock"], margin_start=18)
    row.eb.connect("button-press-event", set_up_dock_tab, preset, preset_name)
    list_box.add(row)

    # row = SideMenuRow(voc["exit-menu"], margin_start=18)
    # row.eb.connect("button-press-event", set_up_bar_tab, preset, preset_name)
    # list_box.add(row)

    row = SideMenuRow(voc["notifications"], margin_start=18)
    row.eb.connect("button-press-event", set_up_notification_tab, preset, preset_name)
    list_box.add(row)

    row = SideMenuRow(voc["gtklock"], margin_start=18)
    row.eb.connect("button-press-event", set_up_gtklock_preset_tab, preset, preset_name)
    list_box.add(row)

    if preset_id == "c":
        row = SideMenuRow(voc["panel-css"], margin_start=18)
        row.eb.connect("button-press-event", set_up_panel_styling_tab, preset, preset_name)
        list_box.add(row)

    global submenus
    submenus.append(list_box)

    list_box.set_selection_mode(Gtk.SelectionMode.NONE)
    return list_box


def hide_submenus():
    for item in submenus:
        if item != current_submenu:
            item.hide()


def toggle_submenu(event_box, event_button, listbox):
    global current_submenu
    current_submenu = listbox
    hide_submenus()
    if not listbox.is_visible():
        listbox.show_all()
        listbox.unselect_all()
    else:
        listbox.hide()


def set_up_screen_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    global update_btn
    content = screen_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_applications_tab(*args, warn=False):
    hide_submenus()
    global content
    content.destroy()
    content = applications_tab(settings, voc, warn)
    grid.attach(content, 1, 0, 1, 1)


def set_up_backup_tab(btn, event, config_home, data_home, backup_configs, backup_data):
    hide_submenus()
    global content
    content.destroy()
    content = backup_tab(config_home, data_home, backup_configs, backup_data, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_autotiling_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = autotiling_tab(settings, outputs, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_keyboard_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = keyboard_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_pointer_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = pointer_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_touchpad_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = touchpad_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_lockscreen_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = lockscreen_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_gtklock_tab(*args):
    hide_submenus()
    global content
    content.destroy()
    content = gtklock_tab(settings, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_drawer_tab(event_box, event_button, preset, preset_name):
    hide_submenus()
    global content
    content.destroy()
    content = drawer_tab(preset, preset_name, outputs, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_dock_tab(event_box, event_button, preset, preset_name):
    hide_submenus()
    global content
    content.destroy()
    content = dock_tab(preset, preset_name, outputs, voc)
    grid.attach(content, 1, 0, 1, 1)


# def set_up_bar_tab(event_box, event_button, preset, preset_name):
#     hide_submenus()
#     global content
#     content.destroy()
#     content = bar_tab(preset, preset_name, voc)
#     grid.attach(content, 1, 0, 1, 1)


def set_up_notification_tab(event_box, event_button, preset, preset_name):
    hide_submenus()
    global content
    content.destroy()
    content = notification_tab(preset, preset_name, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_gtklock_preset_tab(event_box, event_button, preset, preset_name):
    hide_submenus()
    global content
    content.destroy()
    content = gtklock_preset_tab(preset, preset_name, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_panel_styling_tab(event_box, event_button, preset, preset_name):
    hide_submenus()
    global content
    content.destroy()
    content = panel_styling_tab(settings, preset, preset_name, voc)
    grid.attach(content, 1, 0, 1, 1)


def set_up_sys_info_tab(event_box, event_button):
    hide_submenus()
    global content
    content.destroy()
    content = sys_info_tab(voc)
    grid.attach(content, 1, 0, 1, 1)


def on_apply_btn(b):
    f = os.path.join(data_dir, "settings")
    print("Saving {}".format(f))
    save_json(settings, f)

    save_presets()
    presets = {
        "preset-0": preset_0,
        "preset-1": preset_1,
        "preset-2": preset_2,
        "preset-3": preset_3,
        "custom": preset_custom
    }
    preset = presets[settings["panel-preset"]]
    update_swaync_config(preset["swaync-positionX"],
                         preset["swaync-positionY"],
                         preset["swaync-control-center-width"],
                         preset["swaync-notification-window-width"],
                         preset["swaync-mpris"])

    save_includes()


class GUI(object):
    def __init__(self):
        self.label_interface_locale = None
        self.scrolled_window = None
        self.menu = None
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(dir_name, "glade/form.glade"))

        self.window = builder.get_object("window")
        self.window.connect('destroy', Gtk.main_quit)
        self.window.connect("key-release-event", handle_keyboard)

        global grid
        grid = builder.get_object("grid")

        self.build_side_menu()

        self.version = builder.get_object("version-label")
        self.version.set_text("v{}".format(__version__))

        github = builder.get_object("github")
        github.set_markup('<a href="https://github.com/nwg-piotr/nwg-shell-config">GitHub</a>')

        self.cb_show = builder.get_object("show-on-startup")
        self.cb_show.set_active(settings["show-on-startup"])
        self.cb_show.set_label(voc["show-on-startup"])
        self.cb_show.set_tooltip_text(voc["show-on-startup-tooltip"])
        self.cb_show.connect("toggled", set_from_checkbutton, settings, "show-on-startup")

        self.label_interface_locale = builder.get_object("interface-locale")
        self.label_interface_locale.set_text("{}:".format(voc["interface-locale"]))

        self.combo_interface_locale = builder.get_object("combo-interface-locale")
        self.combo_interface_locale.append("auto", "auto")
        self.combo_interface_locale.set_tooltip_text(voc["interface-locale-tooltip"])
        locale_dir = os.path.join(dir_name, "langs")
        entries = os.listdir(locale_dir)
        entries.sort()
        for entry in entries:
            if entry.endswith(".json"):
                loc = entry.split(".")[0]
                self.combo_interface_locale.append(loc, loc)
        if shell_data["interface-locale"]:
            self.combo_interface_locale.set_active_id(shell_data["interface-locale"])
        else:
            self.combo_interface_locale.set_active_id("auto")
        self.combo_interface_locale.connect("changed", set_interface_locale)

        btn_translate = builder.get_object("button-translate")
        icon = Gtk.Image.new_from_icon_name("nwg-shell-translate", Gtk.IconSize.BUTTON)
        btn_translate.set_image(icon)
        btn_translate.set_tooltip_text("Would you like to help with translations?")
        btn_translate.connect("clicked", on_button_translate)

        self.btn_close = builder.get_object("btn-close")
        self.btn_close.set_label(voc["close"])
        self.btn_close.set_tooltip_text(voc["close-tooltip"])
        self.btn_close.connect("clicked", Gtk.main_quit)
        self.btn_close.grab_focus()

        global btn_apply
        btn_apply = builder.get_object("btn-apply")
        btn_apply.set_label(voc["apply"])
        btn_apply.set_tooltip_text(voc["apply-tooltip"])
        btn_apply.connect("clicked", on_apply_btn)

        if settings["night-lat"] == -1 or settings["night-long"] == -1:
            self.tz, self.lat, self.long = get_lat_lon()

    def refresh_bottom_menu_locale(self):
        self.label_interface_locale.set_text("{}:".format(voc["interface-locale"]))
        self.cb_show.set_label(voc["show-on-startup"])
        self.cb_show.set_tooltip_text(voc["show-on-startup-tooltip"])
        self.combo_interface_locale.set_tooltip_text(voc["interface-locale-tooltip"])
        btn_apply.set_label(voc["apply"])
        btn_apply.set_tooltip_text(voc["apply-tooltip"])

        self.btn_close.set_label(voc["close"])
        self.btn_close.set_tooltip_text(voc["close-tooltip"])

    def build_side_menu(self):
        if self.scrolled_window:
            self.scrolled_window.destroy()
        self.scrolled_window = Gtk.ScrolledWindow.new(None, None)
        self.scrolled_window.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.scrolled_window.set_propagate_natural_height(True)
        grid.attach(self.scrolled_window, 0, 0, 1, 1)

        self.menu = side_menu()
        self.scrolled_window.add(self.menu)
        self.scrolled_window.show_all()


def set_interface_locale(combo):
    # selected locale will affect all the nwg-shell components... one day. Let's save it globally.
    if combo.get_active_id() and combo.get_active_id() != "auto":
        shell_data["interface-locale"] = combo.get_active_id()
    else:
        shell_data["interface-locale"] = ""
    save_json(shell_data, os.path.join(get_shell_data_dir(), "data"))

    load_vocabulary()

    # refresh UI on language changed
    global ui
    if ui:
        ui.build_side_menu()
        ui.refresh_bottom_menu_locale()
    # kinda shortcut, but let's just get to the screen tab, instead of remembering which one we were in
    set_up_screen_tab()


def on_button_translate(btn):
    launch(btn, "nwg-shell-translate")


def save_includes():
    cmd_launcher_autostart = ""

    # ~/.config/sway/variables
    variables = []
    if settings["terminal"]:
        variables.append("set $term {}".format(settings["terminal"]))
    if settings["browser"]:
        variables.append("set $browser {}".format(settings["browser"]))
    if settings["file-manager"]:
        variables.append("set $filemanager {}".format(settings["file-manager"]))
    if settings["editor"]:
        variables.append("set $editor {}".format(settings["editor"]))

    if settings["panel-preset"] == "preset-0":
        preset = preset_0
    elif settings["panel-preset"] == "preset-1":
        preset = preset_1
    elif settings["panel-preset"] == "preset-2":
        preset = preset_2
    elif settings["panel-preset"] == "preset-3":
        preset = preset_3
    else:
        preset = preset_custom

    cmd_launcher = "nwg-drawer"
    if preset["launcher-resident"]:
        cmd_launcher += " -r"
    if preset["launcher-columns"]:
        cmd_launcher += " -c {}".format(preset["launcher-columns"])
    if preset["launcher-icon-size"]:
        cmd_launcher += " -is {}".format(preset["launcher-icon-size"])
    if preset["launcher-file-search-columns"]:
        cmd_launcher += " -fscol {}".format(preset["launcher-file-search-columns"])
    if not preset["launcher-search-files"]:
        cmd_launcher += " -nofs"
    if not preset["launcher-categories"]:
        cmd_launcher += " -nocats"
    if preset["launcher-overlay"]:
        cmd_launcher += " -ovl"
    if preset["launcher-gtk-theme"]:
        cmd_launcher += " -g {}".format(preset["launcher-gtk-theme"])
    if preset["launcher-gtk-icon-theme"]:
        # dict: {"theme_name": "folder_name"}
        theme_names = get_icon_themes()
        cmd_launcher += " -i {}".format(theme_names[preset["launcher-gtk-icon-theme"]])

    if "preset-" in settings["panel-preset"]:
        cmd_launcher += " -s {}.css".format(settings["panel-preset"])
    elif preset["launcher-css"]:
        cmd_launcher += " -s {}".format(preset["launcher-css"])

    if settings["terminal"]:
        cmd_launcher += " -term {}".format(settings["terminal"])

    if preset["launcher-force-theme"]:
        cmd_launcher += " -ft"

    if preset["launcher-run-through-compositor"]:
        cmd_launcher += " -wm sway"
    elif preset["launcher-run-through-uwsm"]:
        cmd_launcher += " -wm uwsm"

    if preset["launcher-output"] and preset["launcher-output"] != "Any":
        cmd_launcher += " -o {}".format(preset["launcher-output"])

    if preset["powerbar-on"]:
        if settings["pb-exit"]:
            cmd_launcher += f' -pbexit \'{settings["pb-exit"]}\''
        if settings["pb-lock"]:
            cmd_launcher += f' -pblock \'{settings["pb-lock"]}\''
        if settings["pb-poweroff"]:
            cmd_launcher += f' -pbpoweroff \'{settings["pb-poweroff"]}\''
        if settings["pb-reboot"]:
            cmd_launcher += f' -pbreboot \'{settings["pb-reboot"]}\''
        if settings["pb-sleep"]:
            cmd_launcher += f' -pbsleep \'{settings["pb-sleep"]}\''
        if preset["pb-size"] >= 8:
            cmd_launcher += f' -pbsize {preset["pb-size"]}'
        if preset["pb-icon-theme"]:
            cmd_launcher += ' -pbuseicontheme'

    if preset["launcher-on"]:
        if preset["launcher-resident"]:
            cmd_launcher_autostart = "exec_always {}".format(cmd_launcher)
            variables.append("set $launcher nwg-drawer")
        else:
            variables.append("set $launcher {}".format(cmd_launcher))

    if preset["exit-on"]:
        cmd_exit = "nwg-bar"
        if preset["exit-position"]:
            cmd_exit += " -p {}".format(preset["exit-position"])
        if preset["exit-full"]:
            cmd_exit += " -f"
        if preset["exit-alignment"]:
            cmd_exit += " -a {}".format(preset["exit-alignment"])
        if preset["exit-margin"]:
            cmd_exit += " -mb {} -ml {} -mr {} -mt {}".format(preset["exit-margin"], preset["exit-margin"],
                                                              preset["exit-margin"], preset["exit-margin"])
        if preset["exit-icon-size"]:
            cmd_exit += " -i {}".format(preset["exit-icon-size"])

        if "preset-" in settings["panel-preset"]:
            cmd_exit += " -s {}.css".format(settings["panel-preset"])
        elif preset["exit-css"]:
            cmd_exit += " -s {}".format(preset["exit-css"])
    else:
        cmd_exit = "$launcher"

    variables.append("set $exit {}".format(cmd_exit))

    cmd_dock = "nwg-dock"
    if preset["dock-autohide"]:
        cmd_dock += " -d"
    elif preset["dock-permanent"]:
        cmd_dock += " -r"
    if preset["dock-position"]:
        cmd_dock += " -p {}".format(preset["dock-position"])
    if preset["dock-output"] and preset["dock-output"] != "Any":
        cmd_dock += " -o {}".format(preset["dock-output"])
    if preset["dock-layer"]:
        cmd_dock += " -l {}".format(preset["dock-layer"])
    if preset["dock-full"]:
        cmd_dock += " -f"
    if preset["dock-alignment"]:
        cmd_dock += " -a {}".format(preset["dock-alignment"])
    if preset["dock-margin"]:
        cmd_dock += " -mb {} -ml {} -mr {} -mt {}".format(preset["dock-margin"], preset["dock-margin"],
                                                          preset["dock-margin"], preset["dock-margin"])
    if preset["dock-icon-size"]:
        cmd_dock += " -i {}".format(preset["dock-icon-size"])

    cmd_dock += " -hd {}".format(preset["dock-hotspot-delay"])

    if preset["dock-exclusive"]:
        cmd_dock += " -x"

    if preset["launcher-at-start"]:
        cmd_dock += " -lp start"

    if "preset-" in settings["panel-preset"]:
        cmd_dock += " -s {}.css".format(settings["panel-preset"])
    elif preset["dock-css"]:
        cmd_dock += " -s {}".format(preset["dock-css"])

    if preset["dock-on"] and not preset["dock-autohide"] and not preset["dock-permanent"]:
        variables.append("set $dock {}".format(cmd_dock))

    save_list_to_text_file(variables, os.path.join(config_home, "sway/variables"))

    # ~/.config/sway/autostart
    autostart = ["# This content was generated by nwg-shell-config. Do not modify it manually.\n"]

    # Kill wlsunset. We will need either to restart it or turn it off
    subprocess.call("pkill -f wlsunset", shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
    time.sleep(0.5)

    if settings["night-on"]:
        cmd = "wlsunset -t {} -T {} -l {} -L {} -g {}".format(settings["night-temp-low"],
                                                              settings["night-temp-high"],
                                                              settings["night-lat"],
                                                              settings["night-long"],
                                                              settings["night-gamma"])
        if settings["night-outputs"]:
            for output in settings["night-outputs"]:
                cmd += " -o {}".format(output)

        try:
            subprocess.Popen(cmd, shell=True)
        except Exception as e:
            eprint(e)

        autostart.append("exec {}".format(cmd))

    name = settings["panel-preset"] if not settings["panel-preset"] == "custom" else "style"
    p = os.path.join(config_home, "swaync")
    autostart.append("exec swaync -s {}/{}.css".format(p, name))

    if settings["appindicator"]:
        autostart.append("exec nm-applet --indicator")

    if settings["cliphist"]:
        autostart.append("exec wl-paste --type text --watch cliphist store")
        autostart.append("exec wl-paste --type image --watch cliphist store")

    if settings["autotiling-on"]:
        cmd_autotiling = "exec_always nwg-autotiling"

        if settings["autotiling-workspaces"]:
            cmd_autotiling += " -w {}".format(settings["autotiling-workspaces"])

        autostart.append(cmd_autotiling)

    if cmd_launcher_autostart:
        autostart.append(cmd_launcher_autostart)

    if preset["dock-on"] and (preset["dock-autohide"] or preset["dock-permanent"]):
        autostart.append("exec_always {}".format(cmd_dock))

    cmd_panel = "exec_always nwg-panel"
    if settings["panel-preset"] != "custom":
        cmd_panel += " -c {}".format(settings["panel-preset"])
    elif settings["panel-custom"]:
        cmd_panel += " -c {}".format(settings["panel-custom"])
    if preset["panel-css"]:
        cmd_panel += " -s {}".format(preset["panel-css"])
    autostart.append(cmd_panel)

    if settings["lockscreen-use-settings"]:
        c_sleep = "timeout {} '{}'".format(settings["sleep-timeout"], settings["sleep-cmd"]) if settings[
            "sleep-cmd"] else ""

        c_resume = "resume '{}'".format(settings["resume-cmd"]) if settings["resume-cmd"] else ""
        # Let's make it a bit idiot-proof
        if c_sleep and "dpms off" in settings["sleep-cmd"] and "dpms on" not in settings["resume-cmd"]:
            c_resume = "swaymsg \"output * dpms on\""

        c_after_resume = "after-resume '{}'".format(settings["after-resume"]) if settings["after-resume"] else ""

        c_before_sleep = "before-sleep {}".format(settings["before-sleep"]) if settings[
            "before-sleep"] else ""

        cmd_idle = "exec swayidle timeout {} nwg-lock {} {} {} {}".format(settings["lockscreen-timeout"],
                                                                       c_sleep, c_resume, c_after_resume, c_before_sleep)
        autostart.append(cmd_idle)
        # We can't `exec_always swayidle`, as it would create multiple instances. Let's restart it here.
        subprocess.call("killall swayidle", shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
        subprocess.Popen(cmd_idle, shell=True)

    if settings["update-indicator-on"]:
        autostart.append("exec nwg-update-indicator")

    if settings["screenshot"]:
        autostart.append("exec_always nwg-screenshot-applet")

    if settings["show-on-startup"]:
        autostart.append("exec nwg-shell-config")

    save_list_to_text_file(autostart, os.path.join(config_home, "sway/autostart"))

    # Export keyboard settings
    if settings["keyboard-use-settings"]:
        lines = ['input "type:keyboard" {'] if not settings["keyboard-identifier"] else [
            'input "%s" {' % settings["keyboard-identifier"]]
        if settings["keyboard-xkb-layout"]:
            lines.append('  xkb_layout {}'.format(settings["keyboard-xkb-layout"]))
        if settings["keyboard-xkb-variant"]:
            lines.append('  xkb_variant {}'.format(settings["keyboard-xkb-variant"]))
        lines.append('  repeat_delay {}'.format(settings["keyboard-repeat-delay"]))
        lines.append('  repeat_rate {}'.format(settings["keyboard-repeat-rate"]))
        lines.append('  xkb_capslock {}'.format(settings["keyboard-xkb-capslock"]))
        lines.append('  xkb_numlock {}'.format(settings["keyboard-xkb-numlock"]))
        if settings["keyboard-custom-name"] and settings["keyboard-custom-value"]:
            lines.append('  {} {}'.format(settings["keyboard-custom-name"], settings["keyboard-custom-value"]))
        lines.append('}')

        if preset["launcher-super-key"]:
            lines.append('bindsym --release Super_L exec $launcher')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/keyboard"))
    else:
        save_list_to_text_file([""], os.path.join(config_home, "sway/keyboard"))

    # Export pointer device settings
    if settings["pointer-use-settings"]:
        lines = ['input "type:pointer" {'] if not settings["pointer-identifier"] else [
            'input "%s" {' % settings["pointer-identifier"]]
        lines.append('  natural_scroll {}'.format(settings["pointer-natural-scroll"]))
        lines.append('  scroll_factor {}'.format(settings["pointer-scroll-factor"]))
        lines.append('  left_handed {}'.format(settings["pointer-left-handed"]))
        if settings["pointer-custom-name"] and settings["pointer-custom-value"]:
            lines.append('  {} {}'.format(settings["keyboard-custom-name"], settings["keyboard-custom-value"]))
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/pointer"))
    else:
        save_list_to_text_file([""], os.path.join(config_home, "sway/pointer"))

    # Export touchpad settings
    if settings["touchpad-use-settings"]:
        lines = ['input "type:touchpad" {'] if not settings["touchpad-identifier"] else [
            'input "%s" {' % settings["touchpad-identifier"]]
        lines.append('  pointer_accel {}'.format(settings["touchpad-pointer-accel"]))
        lines.append('  natural_scroll {}'.format(settings["touchpad-natural-scroll"]))
        lines.append('  scroll_factor {}'.format(settings["touchpad-scroll-factor"]))
        lines.append('  scroll_method {}'.format(settings["touchpad-scroll-method"]))
        lines.append('  click_method {}'.format(settings["touchpad-click-method"]))
        lines.append('  left_handed {}'.format(settings["touchpad-left-handed"]))
        lines.append('  tap {}'.format(settings["touchpad-tap"]))
        lines.append('  tap_button_map {}'.format(settings["touchpad-tap-button-map"]))
        lines.append('  drag {}'.format(settings["touchpad-drag"]))
        lines.append('  drag_lock {}'.format(settings["touchpad-drag-lock"]))
        lines.append('  dwt {}'.format(settings["touchpad-dwt"]))
        lines.append('  middle_emulation {}'.format(settings["touchpad-middle-emulation"]))
        if settings["touchpad-custom-name"] and settings["touchpad-custom-value"]:
            lines.append('  {} {}'.format(settings["touchpad-custom-name"], settings["touchpad-custom-value"]))
        lines.append('}')

        save_list_to_text_file(lines, os.path.join(config_home, "sway/touchpad"))
    else:
        save_list_to_text_file([""], os.path.join(config_home, "sway/touchpad"))

    reload()


def reload():
    name = settings["panel-preset"] if not settings["panel-preset"] == "custom" else "style"
    p = os.path.join(config_home, "swaync")
    swaync_daemon = "swaync -s {}/{}.css &".format(p, name)

    for cmd in ["pkill -f autotiling",
                "pkill -f nwg-drawer",
                "pkill -f nwg-dock",
                "pkill -f nwg-bar",
                "pkill -f swaync",
                swaync_daemon,
                "swaync-client --reload-config",
                "swaymsg reload"]:
        subprocess.call(cmd, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)

    if settings["update-indicator-on"]:
        launch(None, "nwg-update-indicator")
    else:
        subprocess.call("pkill -f nwg-update-indicator", shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)

    if not settings["screenshot"]:
        subprocess.call("pkill -f nwg-screenshot", shell=True, stdout=subprocess.DEVNULL,
                        stderr=subprocess.STDOUT)


def load_settings():
    settings_file = os.path.join(data_dir, "settings")
    defaults = {
        "keyboard-layout": "us",
        "autotiling-workspaces": "",
        "autotiling-on": True,
        "autotiling-limit": False,
        "autotiling-output-limits": {},
        "autotiling-output-splitwidths": {},
        "autotiling-output-splitheights": {},
        "appindicator": True,
        "cliphist": True,
        "night-lat": -1,
        "night-long": -1,
        "night-temp-low": 4500,
        "night-temp-high": 6500,
        "night-gamma": 1.0,
        "night-on": True,
        "night-outputs": [],
        "terminal": "",
        "file-manager": "",
        "editor": "",
        "browser": "",

        "pb-exit": "swaymsg exit",
        "pb-lock": "nwg-lock",
        "pb-poweroff": "systemctl -i poweroff",
        "pb-reboot": "systemctl reboot",
        "pb-sleep": "systemctl suspend",

        "panel-preset": "preset-0",
        "panel-custom": "",
        "screenshot": True,
        "show-on-startup": True,
        "keyboard-use-settings": True,
        "keyboard-identifier": "",
        "keyboard-xkb-layout": "us",
        "keyboard-xkb-variant": "",
        "keyboard-repeat-delay": 300,
        "keyboard-repeat-rate": 40,
        "keyboard-xkb-capslock": "disabled",
        "keyboard-xkb-numlock": "disabled",
        "keyboard-custom-name": "",
        "keyboard-custom-value": "",
        "pointer-use-settings": True,
        "pointer-identifier": "",
        "pointer-accel-profile": "flat",
        "pointer-pointer-accel": 0.0,
        "pointer-natural-scroll": "disabled",
        "pointer-scroll-factor": 1.0,
        "pointer-left-handed": "disabled",
        "pointer-custom-name": "",
        "pointer-custom-value": "",
        "touchpad-use-settings": True,
        "touchpad-identifier": "",
        "touchpad-accel-profile": "flat",
        "touchpad-pointer-accel": 0.0,
        "touchpad-natural-scroll": "disabled",
        "touchpad-scroll-factor": 1.0,
        "touchpad-scroll-method": "two_finger",
        "touchpad-left-handed": "disabled",
        "touchpad-tap": "enabled",
        "touchpad-tap-button-map": "lrm",
        "touchpad-drag": "enabled",
        "touchpad-drag-lock": "disabled",
        "touchpad-dwt": "enabled",
        "touchpad-middle-emulation": "enabled",
        "touchpad-click-method": "button_areas",
        "touchpad-custom-name": "",
        "touchpad-custom-value": "",
        "lockscreen-use-settings": True,
        "lockscreen-locker": "gtklock",  # swaylock | gtklock
        "lockscreen-background-source": "local",  # unsplash | local
        "lockscreen-custom-cmd": "",
        "lockscreen-timeout": 1200,
        "sleep-cmd": 'swaymsg "output * dpms off"',
        "sleep-timeout": 1800,
        "resume-cmd": 'swaymsg "output * dpms on"',
        "after-resume": 'swaymsg "output * enable"',
        "before-sleep": "",
        "backgrounds-custom-path": "",
        "backgrounds-use-custom-path": False,
        "background-dirs": ["/usr/share/backgrounds/nwg-shell"],
        "background-dirs-once-set": False,
        "unsplash-width": 1920,
        "unsplash-height": 1080,
        "unsplash-keywords": ["nature", "water", "landscape"],
        "wallhaven-ratio": "16x9,16x10",
        "wallhaven-atleast": "1920x1080",
        "wallhaven-api-key": "",
        "wallhaven-tags": ["nature", "landscape"],
        "help-font-size": 12,
        "help-layer-shell": True,
        "help-keyboard": False,
        "gtklock-disable-input-inhibitor": False,
        "gtklock-idle-timeout": 10,
        "gtklock-logout-command": "swaymsg exit",
        "gtklock-playerctl": False,
        "gtklock-powerbar": False,
        "gtklock-poweroff-command": "systemctl -i poweroff",
        "gtklock-reboot-command": "systemctl reboot",
        "gtklock-suspend-command": "systemctl suspend",
        "gtklock-time-format": "%H:%M:%S",
        "gtklock-userinfo": False,
        "gtklock-userswitch-command": "",
        "update-indicator-on": False,
        "update-indicator-interval": 30,
        "update-command": "nwg-system-update"
    }
    global settings
    if os.path.isfile(settings_file):
        print("Loading settings from {}".format(settings_file))
        settings = load_json(settings_file)
        missing = 0
        for key in defaults:
            if key not in settings:
                settings[key] = defaults[key]
                print("'{}' key missing from settings, adding '{}'".format(key, defaults[key]))
                missing += 1
        if missing > 0:
            print("{} missing config key(s) substituted. Saving {}".format(missing, settings_file))
            save_json(settings, settings_file)
    else:
        print("ERROR: failed loading settings, creating {}".format(settings_file), file=sys.stderr)
        save_json(defaults, settings_file)


def load_presets():
    global preset_0
    preset_0 = load_preset("preset-0")
    global preset_1
    preset_1 = load_preset("preset-1")
    global preset_2
    preset_2 = load_preset("preset-2")
    global preset_3
    preset_3 = load_preset("preset-3")
    global preset_custom
    preset_custom = load_preset("custom")


def load_preset(file_name):
    defaults = {
        "panel-css": "",
        "launcher-at-start": False,
        "launcher-columns": 6,
        "launcher-icon-size": 64,
        "launcher-file-search-columns": 2,
        "launcher-search-files": True,
        "launcher-categories": True,
        "launcher-resident": False,
        "launcher-overlay": False,
        "launcher-css": "",
        "launcher-on": True,
        "launcher-gtk-theme": "",
        "launcher-gtk-icon-theme": "",
        "launcher-force-theme": False,
        "launcher-run-through-compositor": False,
        "launcher-run-through-uwsm": False,
        "launcher-super-key": False,
        "launcher-output": "Any",
        "exit-position": "center",
        "exit-full": False,
        "exit-alignment": "middle",
        "exit-margin": 0,
        "exit-icon-size": 48,
        "exit-css": "",
        "exit-on": True,
        "dock-position": "bottom",
        "dock-output": "Any",
        "dock-full": False,
        "dock-autohide": False,
        "dock-permanent": False,
        "dock-exclusive": False,
        "dock-alignment": "center",
        "dock-layer": "overlay",
        "dock-margin": 0,
        "dock-icon-size": 48,
        "dock-hotspot-delay": 20,
        "dock-startup-delay": 0,
        "dock-css": "",
        "dock-on": False,
        "dock-nolauncher": False,
        "swaync-positionX": "right",
        "swaync-positionY": "top",
        "swaync-control-center-width": 500,
        "swaync-notification-window-width": 500,
        "swaync-mpris": False,
        "gtklock-userinfo-round-image": True,
        "gtklock-userinfo-vertical-layout": True,
        "gtklock-userinfo-under-clock": False,
        "gtklock-powerbar-show-labels": True,
        "gtklock-powerbar-linked-buttons": False,
        "gtklock-playerctl-art-size": 64,
        "gtklock-playerctl-position": "top-right",
        "gtklock-playerctl-show-hidden": True,
        "powerbar-on": True,
        "pb-icon-theme": False,
        "pb-size": 48
    }
    preset_file = os.path.join(data_dir, file_name)
    if os.path.isfile(preset_file):
        print("Loading preset from {}".format(preset_file))
        preset = load_json(preset_file)
        missing = 0
        for key in defaults:
            if key not in preset:
                preset[key] = defaults[key]
                print("'{}' key missing from preset, adding '{}'".format(key, defaults[key]))
                missing += 1
        if missing > 0:
            print("Saving {}".format(preset_file))
            save_json(preset, preset_file)

        return preset
    else:
        print("ERROR: failed loading preset, creating {}".format(preset_file), file=sys.stderr)
        save_json(defaults, preset_file)

        return {}


def save_presets():
    global preset_0, preset_1, preset_2, preset_3, preset_custom

    f = os.path.join(data_dir, "preset-0")
    print("Saving {}".format(f))
    save_json(preset_0, f)

    f = os.path.join(data_dir, "preset-1")
    print("Saving {}".format(f))
    save_json(preset_1, f)

    f = os.path.join(data_dir, "preset-2")
    print("Saving {}".format(f))
    save_json(preset_2, f)

    f = os.path.join(data_dir, "preset-3")
    print("Saving {}".format(f))
    save_json(preset_3, f)

    f = os.path.join(data_dir, "custom")
    print("Saving {}".format(f))
    save_json(preset_custom, f)


def update_swaync_config(pos_x, pos_y, cc_width, window_width, mpris):
    settings_file = os.path.join(config_home, "swaync/config.json")
    if os.path.isfile(settings_file):
        print("Loading swaync settings from {}".format(settings_file))
        swaync_settings = load_json(settings_file)

        # Check if some new keys appeared
        defaults = load_json("/etc/xdg/swaync/config.json")

        for key in defaults:
            if key not in swaync_settings:
                swaync_settings[key] = defaults[key]

        swaync_settings["positionX"] = pos_x
        swaync_settings["positionY"] = pos_y
        swaync_settings["control-center-width"] = cc_width
        swaync_settings["notification-window-width"] = window_width
    else:
        swaync_settings = {
            "positionX": pos_x,
            "positionY": pos_y,
            "control-center-width": cc_width,
            "notification-window-width": window_width
        }

    if mpris:
        if "mpris" not in swaync_settings["widgets"]:
            swaync_settings["widgets"].append("mpris")
    else:
        if "mpris" in swaync_settings["widgets"]:
            swaync_settings["widgets"].remove("mpris")

    print("Saving swaync settings to {}".format(settings_file))
    save_json(swaync_settings, settings_file)


def load_vocabulary():
    global voc
    # basic vocabulary (for en_US)
    voc = load_json(os.path.join(dir_name, "langs", "en_US.json"))
    if not voc:
        eprint("Failed loading vocabulary")
        sys.exit(1)

    lang = os.getenv("LANG").split(".")[0] if not shell_data["interface-locale"] else shell_data["interface-locale"]
    # translate if necessary
    if lang != "en_US":
        loc_file = os.path.join(dir_name, "langs", "{}.json".format(lang))
        if os.path.isfile(loc_file):
            # localized vocabulary
            loc = load_json(loc_file)
            if not loc:
                eprint("Failed loading translation into '{}'".format(lang))
            else:
                for key in loc:
                    voc[key] = loc[key]


def load_settings_save_includes():
    global data_dir
    data_dir = get_data_dir()
    load_presets()
    load_settings()
    save_includes()
    sys.exit(0)


def restore_backup(b_path, voc):
    if b_path.startswith("~/"):
        b_path = os.path.join(os.getenv("HOME"), b_path[2:])
    unpack_from_path(b_path)
    restore_from_tmp(None, None, voc)


def main():
    global data_dir
    data_dir = get_data_dir()

    parser = argparse.ArgumentParser()
    parser.add_argument("-v",
                        "--version",
                        action="version",
                        version="%(prog)s version {}".format(__version__),
                        help="display version information")

    parser.add_argument("-r",
                        "--restore",
                        action="store_true",
                        help="restore default presets")

    parser.add_argument("-b",
                        "--restore_backup",
                        type=str,
                        default="",
                        help="restore all configs from a backup file (given path)")

    parser.add_argument("-s",
                        "--save",
                        action="store_true",
                        help="load settings & Save includes (for use w/ external scripts)")

    parser.parse_args()
    args = parser.parse_args()

    if args.save:
        init_files(os.path.join(dir_name, "shell"), data_dir)
        load_settings_save_includes()

    if not os.getenv("SWAYSOCK"):
        eprint("SWAYSOCK not found, terminating")
        sys.exit(1)

    GLib.set_prgname('nwg-shell-config')

    global outputs
    outputs = list_outputs()

    print("Outputs: {}".format(outputs))
    print("Data dir: {}".format(data_dir))
    print("Config home: {}".format(config_home))

    load_vocabulary()

    if args.restore:
        if input("\nRestore default presets? y/N ").upper() == "Y":
            init_files(os.path.join(dir_name, "shell"), data_dir, overwrite=True)
            sys.exit(0)
    elif args.restore_backup:
        restore_backup(args.restore_backup, voc)
        sys.exit(0)
    else:
        # initialize missing own data files
        init_files(os.path.join(dir_name, "shell"), data_dir)

    check_updates()

    for folder in ["nwg-look", "nwg-shell", "nwg-shell-config"]:
        src = os.path.join("/etc/skel/.local/share", folder)
        dst = os.path.join(data_home, folder)
        if os.path.exists(src) and not os.path.exists(dst):
            os.mkdir(dst)
            print(dst, "missing, initializing")
            init_files(src, dst)

    load_settings()

    load_presets()

    global ui
    ui = GUI()

    screen = Gdk.Screen.get_default()
    provider = Gtk.CssProvider()
    style_context = Gtk.StyleContext()
    style_context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
    css = b"""
            button#app-btn { padding: 6px 2px 6px 2px; border: none }
            * { outline: none }
            """
    provider.load_from_data(css)

    if not settings["terminal"] or not settings["file-manager"] or not settings["editor"] or not settings["browser"]:
        set_up_applications_tab(warn=True)
    else:
        set_up_screen_tab()

    ui.window.show_all()
    hide_submenus()

    catchable_sigs = set(signal.Signals) - {signal.SIGKILL, signal.SIGSTOP}
    for sig in catchable_sigs:
        signal.signal(sig, signal_handler)

    Gtk.main()


if __name__ == '__main__':
    sys.exit(main())



try:
    from importlib import metadata
except ImportError:
    import importlib_metadata as metadata

try:
    __version__ = metadata.version("nwg-shell-config")
except Exception:
    __version__ = "unknown"

# Shell versions that need to trigger upgrade
__need_update__ = ["0.3.0", "0.3.4", "0.5.0"]



#!/usr/bin/env python3

"""
nwg-shell helper script to display system help window
Copyright (c) 2022-2025 Piotr Miller
e-mail: nwg.piotr@gmail.com
Project: https://github.com/nwg-piotr/nwg-shell
License: MIT
"""

import os
import signal
import sys

import gi

gi.require_version('Gtk', '3.0')
try:
    gi.require_version('GtkLayerShell', '0.1')
except ValueError:
    raise RuntimeError('\n\n' +
                       'If you haven\'t installed GTK Layer Shell, you need to point Python to the\n' +
                       'library by setting GI_TYPELIB_PATH and LD_LIBRARY_PATH to <build-dir>/src/.\n' +
                       'For example you might need to run:\n\n' +
                       'GI_TYPELIB_PATH=build/src LD_LIBRARY_PATH=build/src python3 ' + ' '.join(sys.argv))

dir_name = os.path.dirname(__file__)

from gi.repository import Gtk, Gdk, GtkLayerShell

from nwg_shell_config.tools import temp_dir, init_files, get_data_dir, load_json, load_text_file, save_string, eprint, \
    check_key

data_dir = get_data_dir()

if os.getenv("SWAYSOCK"):
    settings = load_json(os.path.join(data_dir, "settings"))
elif os.getenv("HYPRLAND_INSTANCE_SIGNATURE"):
    settings = load_json(os.path.join(data_dir, "settings-hyprland"))
else:
    eprint("This program only works on sway or Hyprland, terminating.")
    sys.exit(1)

defaults = {
    "help-font-size": 14,
    "help-layer-shell": True,
    "help-keyboard": False
}
for key in defaults:
    check_key(settings, key, defaults[key])


def signal_handler(sig, frame):
    if sig == 2 or sig == 15:
        desc = {2: "SIGINT", 15: "SIGTERM"}
        print("Terminated with {}".format(desc[sig]))
        Gtk.main_quit()


def handle_keyboard(win, event):
    if event.type == Gdk.EventType.KEY_RELEASE and event.keyval == Gdk.KEY_Escape:
        win.destroy()


def main():
    # Try and kill already running instance, if any
    pid_file = os.path.join(temp_dir(), "nwg-help.pid")
    if os.path.isfile(pid_file):
        try:
            pid = int(load_text_file(pid_file))
            os.kill(pid, signal.SIGINT)
            print("Running instance killed, PID {}".format(pid))
            sys.exit(0)
        except ProcessLookupError:
            pass
    save_string(str(os.getpid()), pid_file)

    if os.getenv("SWAYSOCK"):
        content_path = os.path.join(data_dir, "help-sway.pango")
    elif os.getenv("HYPRLAND_INSTANCE_SIGNATURE"):
        content_path = os.path.join(data_dir, "help-hyprland.pango")
    else:
        eprint("This program only works on sway or Hyprland, terminating.")
        sys.exit(1)

    if not os.path.isfile(content_path):
        init_files(os.path.join(dir_name, "shell"), data_dir)
    content = load_text_file(content_path)

    window = Gtk.Window.new(Gtk.WindowType.POPUP)

    if settings["help-layer-shell"]:
        GtkLayerShell.init_for_window(window)
        GtkLayerShell.set_layer(window, GtkLayerShell.Layer.TOP)
        GtkLayerShell.set_exclusive_zone(window, 0)
        if settings["help-keyboard"]:
            GtkLayerShell.set_keyboard_mode(window, GtkLayerShell.KeyboardMode.ON_DEMAND)

    window.connect('destroy', Gtk.main_quit)
    window.connect("key-release-event", handle_keyboard)

    scrolled_window = Gtk.ScrolledWindow.new(None, None)
    scrolled_window.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled_window.set_propagate_natural_height(True)
    window.add(scrolled_window)

    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
    box.set_property("margin", 12)
    scrolled_window.add(box)

    label = Gtk.Label()
    label.set_markup(content)

    box.pack_start(label, False, False, 0)

    screen = Gdk.Screen.get_default()
    provider = Gtk.CssProvider()
    style_context = Gtk.StyleContext()
    style_context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
    css = b""" * { border-radius: 0px } """
    css += b""" window { border: solid 1px; border-color: #000 } """
    font_string = "label { font-size: %dpx }" % settings["help-font-size"]
    css += str.encode(font_string)
    provider.load_from_data(css)

    window.show_all()

    if settings["help-layer-shell"]:
        window.set_size_request(0, window.get_allocated_width() * 1.9)

    catchable_sigs = set(signal.Signals) - {signal.SIGKILL, signal.SIGSTOP}
    for sig in catchable_sigs:
        signal.signal(sig, signal_handler)

    Gtk.main()


if __name__ == "__main__":
    sys.exit(main())
